import subprocess


def __run(args: str):
    """
    Runs git commands
    :param args: any git param
    :return: system call result
    """
    stdout = subprocess.check_output(["git"] + args.split())
    return stdout.decode()


def _list_merged_branchs():
    """
    List merged branchs
    :return:
    """
    out = __run("branch --no-color --merged")
    return [
        b.strip("* ")
        for b in out.splitlines()
        if b.strip("* ") not in ("master", "develop")
    ]


def _delete_branch(name: str):
    __run(f"branch -d {name}")


def _delete_merged_branchs():
    for branch in _list_merged_branchs():
        _delete_branch(branch)


def _fetch_remote_changes():
    __run("fetch --all")


def _get_current_branch():
    out = __run("branch --show-current")
    return out.strip()


def create_new_feature(name: str):
    __run("checkout develop")
    __run("pull origin develop")
    _fetch_remote_changes()
    _delete_merged_branchs()
    __run(f"branch -b feature/{name}")
    return _get_current_branch()


def create_new_hotfix(name: str):
    __run("checkout master")
    __run("pull origin master")
    _fetch_remote_changes()
    _delete_merged_branchs()
    __run(f"branch -b hotfix/{name}")
    return _get_current_branch()


def create_new_release(version: str):
    name = f"v{version.replace('v', '')}"
    __run("checkout develop")
    __run("pull origin develop")
    _fetch_remote_changes()
    _delete_merged_branchs()
    __run(f"branch -b release/{name}")
    return _get_current_branch()


def publish_remote():
    current_branch = _get_current_branch()
    __run(f"push origin {current_branch}")
    return current_branch



import typer

from git_flow_wrapper import git_commands

app = typer.Typer()
new_app = typer.Typer()
app.add_typer(new_app, name="new")


@new_app.command("feature")
def new_feature(name: str):
    """
    Creates a new feature
    :param name: name
    :return:
    """
    feature = git_commands.create_new_feature(name)
    typer.echo(f"Feature `{feature}` created.")


@new_app.command("hotfix")
def new_hotfix(name: str):
    """
    Creates a new hotfix
    :param name: name
    :return:
    """
    hotfix = git_commands.create_new_hotfix(name)
    typer.echo(f"Hotfix `{hotfix}` created.")


@new_app.command("release")
def new_release(version: str):
    """
    Creates a new release
    :param version: x.x
    :return:
    """
    release = git_commands.create_new_release(version)
    typer.echo(f"Release `{release}` created.")


@app.command()
def publish():
    """Publishes the current branch"""
    branch = git_commands.publish_remote()
    typer.echo(f"Branch `{branch}` published.")


def main():
    app()
